;; create calendar and mh-rmail screens.

(defun make-calendar-window ()
  (interactive)
  (setq tmp-screen (current-screen))
  (select-screen 
   (create-screen 
    "*scratch*" 
    (list 
     (cons 'title "Calendar"))))
  (calendar)
  (mark-diary-entries)
  (sit-for 0)
  (select-screen tmp-screen))

(defun make-mh-window ()
  (interactive)
  (setq tmp-screen (current-screen))
  (select-screen 
   (create-screen 
    "*scratch*"
    (list (cons 'title "Mh-Rmail")
          (cons 'geometry "80x60"))))
  (mh-rmail)
  (mh-rescan-folder 'all)
  (sit-for 0)
  (select-screen tmp-screen))
