;;; Copyright (C) 1991 Christopher J. Love
;;;
;;; This file is for use with Epoch, a modified version of GNU Emacs.
;;; Requires Epoch 4.0 or later.
;;;
;;; This code is distributed in the hope that it will be useful,
;;; bute WITHOUT ANY WARRANTY. No author or distributor accepts
;;; responsibility to anyone for the consequences of using this code
;;; or for whether it serves any particular purpose or works at all,
;;; unless explicitly stated in a written agreement.
;;;
;;; Everyone is granted permission to copy, modify and redistribute
;;; this code, but only under the conditions described in the
;;; GNU Emacs General Public License, except the original author nor his
;;; agents are bound by the License in their use of this code.
;;; (These special rights for the author in no way restrict the rights of
;;;  others given in the License or this prologue)
;;; A copy of this license is supposed to have been given to you along
;;; with Epoch so you can know your rights and responsibilities. 
;;; It should be in a file named COPYING.  Among other things, the
;;; copyright notice and this notice must be preserved on all copies. 
;;;
;;; $Revision: 1.4 $
;;; $Source: /import/kaplan/stable/distrib/epoch-4.1/epoch-lisp/RCS/button.el,v $
;;; $Date: 92/02/28 10:56:41 $
;;; $Author: love $
;;;
;;; button.el - provide wrappers for zone functions.  Do this for
;;;		compatibility with existing packages.
;;;
(provide 'button)

(fset 'move-button (symbol-function 'epoch::move-zone))
(fset 'make-button (symbol-function 'epoch::make-zone))
(fset 'button-style (symbol-function 'epoch::zone-style))
(fset 'button-data (symbol-function 'epoch::zone-data))
(fset 'button-start (symbol-function 'epoch::zone-start))
(fset 'button-end (symbol-function 'epoch::zone-end))
(fset 'button-at (symbol-function 'epoch::zone-at))
(fset 'buttons-at (symbol-function 'epoch::zones-at))
(fset 'delete-button (symbol-function 'epoch::delete-zone))
(fset 'delete-button-at (symbol-function 'epoch::delete-zone-at))
(fset 'buttonp (symbol-function 'epoch::zonep))
(fset 'button-list (symbol-function 'epoch::zone-list))
(fset 'buttons-in-region (symbol-function 'epoch::zones-in-region))
(fset 'button-buffer (symbol-function 'epoch::zone-buffer))
(fset 'set-button-style (symbol-function 'epoch::set-zone-style))
(fset 'set-button-data (symbol-function 'epoch::set-zone-data))
(fset 'button-read-only-p (symbol-function 'epoch::zone-read-only-p))
(fset 'set-button-read-only (symbol-function 'epoch::set-zone-read-only))
(fset 'button-transient-p (symbol-function 'epoch::zone-transient-p))
(fset 'set-button-transient (symbol-function 'epoch::set-zone-transient))
(fset 'clear-buttons (symbol-function 'epoch::clear-zones))

(fset 'add-button (symbol-function 'epoch::add-zone))
(fset 'add-read-only-button (symbol-function 'add-read-only-zone))
(if (fboundp 'add-graphic-zone)
  (fset 'add-graphic-button (symbol-function 'add-graphic-zone))
)

(fset 'epoch::move-button (symbol-function 'epoch::move-zone))
(fset 'epoch::make-button (symbol-function 'epoch::make-zone))
(fset 'epoch::button-style (symbol-function 'epoch::zone-style))
(fset 'epoch::button-data (symbol-function 'epoch::zone-data))
(fset 'epoch::button-start (symbol-function 'epoch::zone-start))
(fset 'epoch::button-end (symbol-function 'epoch::zone-end))
(fset 'epoch::button-at (symbol-function 'epoch::zone-at))
(fset 'epoch::buttons-at (symbol-function 'epoch::zones-at))
(fset 'epoch::delete-button (symbol-function 'epoch::delete-zone))
(fset 'epoch::delete-button-at (symbol-function 'epoch::delete-zone-at))
(fset 'epoch::buttonp (symbol-function 'epoch::zonep))
(fset 'epoch::button-list (symbol-function 'epoch::zone-list))
(fset 'epoch::buttons-in-region (symbol-function 'epoch::zones-in-region))
(fset 'epoch::read-only-region-p (symbol-function 'epoch::read-only-region-p))
(fset 'epoch::button-buffer (symbol-function 'epoch::zone-buffer))
(fset 'epoch::set-button-style (symbol-function 'epoch::set-zone-style))
(fset 'epoch::set-button-data (symbol-function 'epoch::set-zone-data))
(fset 'epoch::button-read-only-p (symbol-function 'epoch::zone-read-only-p))
(fset 'epoch::set-button-read-only (symbol-function 'epoch::set-zone-read-only))
(fset 'epoch::button-transient-p (symbol-function 'epoch::zone-transient-p))
(fset 'epoch::set-button-transient (symbol-function 'epoch::set-zone-transient))
(fset 'epoch::clear-buttons (symbol-function 'epoch::clear-zones))

(fset 'epoch::add-button (symbol-function 'epoch::add-zone))
(fset 'epoch::add-read-only-button (symbol-function 'add-read-only-zone))
(if (fboundp 'add-graphic-zone)
  (fset 'epoch::add-graphic-button (symbol-function 'add-graphic-zone))
)
